@@ -63,6 +63,9 @@ static void	gpk_check_update_libnotify_cb (NotifyNotification *notification, gch
 /* the amount of time after ::UpdatesChanged we refresh the update list */
 #define GPK_CHECK_UPDATE_UPDATES_CHANGED_TIMEOUT	3 /* seconds */
 
+/* the amoutn of time we check for updates after an UpdateSystem call failed */
+#define	GPK_CHECK_UPDATE_FAILED_TASK_RECHECK_DELAY	60*60 /* seconds */
+
 struct GpkCheckUpdatePrivate
 {
 	GtkStatusIcon		*status_icon;
@@ -472,7 +475,7 @@ gpk_check_update_update_system_finished_cb (PkTask *task, GAsyncResult *res, Gpk
 
 		/* we failed, so re-get the update list */
 		gpk_check_update_set_icon_name (cupdate, NULL);
-		g_timeout_add_seconds (2, (GSourceFunc) gpk_check_update_get_updates_post_update_cb, cupdate);
+		g_timeout_add_seconds (GPK_CHECK_UPDATE_FAILED_TASK_RECHECK_DELAY, (GSourceFunc) gpk_check_update_get_updates_post_update_cb, cupdate);
 		goto out;
 	}
 
@@ -587,11 +590,10 @@ gpk_check_update_libnotify_cb (NotifyNotification *notification, gchar *action,
  * gpk_check_update_critical_updates_warning:
  **/
 static void
-gpk_check_update_critical_updates_warning (GpkCheckUpdate *cupdate, const gchar *details, GPtrArray *array)
+gpk_check_update_critical_updates_warning (GpkCheckUpdate *cupdate, GPtrArray *array)
 {
 	const gchar *title;
-	gchar *message;
-	GString *string;
+	const gchar *message;
 	gboolean ret;
 	GError *error = NULL;
 	NotifyNotification *notification;
@@ -618,14 +620,9 @@ gpk_check_update_critical_updates_warning (GpkCheckUpdate *cupdate, const gchar
 	/* TRANSLATORS: title in the libnotify popup */
 	title = ngettext ("Security update available", "Security updates available", array->len);
 
-	/* format message text */
-	string = g_string_new ("");
 	/* TRANSLATORS: message when there are security updates */
-	g_string_append (string, ngettext ("The following important update is available for your computer:",
-					   "The following important updates are available for your computer:", array->len));
-	g_string_append (string, "\n\n");
-	g_string_append (string, details);
-	message = g_string_free (string, FALSE);
+	message = ngettext ("An important update is available for your computer:",
+			    "Important updates are available for your computer:", array->len);
 
 	/* close any existing notification */
 	if (cupdate->priv->notification_updates_available != NULL) {
@@ -652,8 +649,6 @@ gpk_check_update_critical_updates_warning (GpkCheckUpdate *cupdate, const gchar
 	}
 	/* track so we can prevent doubled notifications */
 	cupdate->priv->notification_updates_available = notification;
-
-	g_free (message);
 }
 
 /**
@@ -848,21 +843,14 @@ gpk_check_update_get_updates_finished_cb (GObject *object, GAsyncResult *res, Gp
 	GError *error = NULL;
 	PkPackage *item;
 	guint i;
-	guint more;
-	guint showing = 0;
 	gboolean ret;
-	GString *status_security = NULL;
 	GString *status_tooltip = NULL;
 	GpkUpdateEnum update;
 	GPtrArray *security_array = NULL;
 	const gchar *icon;
 	gchar **package_ids;
-	gchar **split;
 	GPtrArray *array = NULL;
 	PkError *error_code = NULL;
-	PkInfoEnum info;
-	gchar *package_id = NULL;
-	gchar *summary = NULL;
 
 	/* get the results */
 	results = pk_client_generic_finish (PK_CLIENT(client), res, &error);
@@ -900,64 +888,21 @@ gpk_check_update_get_updates_finished_cb (GObject *object, GAsyncResult *res, Gp
 	}
 
 	/* we have updates to process */
-	status_security = g_string_new ("");
 	status_tooltip = g_string_new ("");
 	security_array = g_ptr_array_new_with_free_func (g_free);
 
 	/* find the security updates first */
 	for (i=0; i<array->len; i++) {
 		item = g_ptr_array_index (array, i);
-		g_object_get (item,
-			      "info", &info,
-			      "package-id", &package_id,
-			      NULL);
-		if (info == PK_INFO_ENUM_SECURITY) {
-			/* add to array */
-			g_ptr_array_add (security_array, g_strdup (package_id));
-		}
-		g_free (package_id);
-	}
-
-	/* get the security update text */
-	for (i=0; i<array->len; i++) {
-		item = g_ptr_array_index (array, i);
-		g_object_get (item,
-			      "info", &info,
-			      NULL);
-		if (info != PK_INFO_ENUM_SECURITY)
+		if (pk_package_get_info (item) != PK_INFO_ENUM_SECURITY)
 			continue;
-
-		/* get more data */
-		g_object_get (item,
-			      "package-id", &package_id,
-			      "summary", &summary,
-			      NULL);
-
-		/* don't use a huge notification that won't fit on the screen */
-		split = pk_package_id_split (package_id);
-		g_string_append_printf (status_security, "<b>%s</b> - %s\n", split[PK_PACKAGE_ID_NAME], summary);
-		g_strfreev (split);
-		if (++showing == GPK_CHECK_UPDATE_MAX_NUMBER_SECURITY_ENTRIES) {
-			more = security_array->len - showing;
-			if (more > 0) {
-				/* TRANSLATORS: we have a notification that won't fit, so append on how many other we are not showing */
-				g_string_append_printf (status_security, ngettext ("and %d other security update",
-										   "and %d other security updates", more), more);
-				g_string_append (status_security, "...\n");
-			}
-			break;
-		}
-		g_free (package_id);
-		g_free (summary);
+		g_ptr_array_add (security_array, g_strdup (pk_package_get_id (item)));
 	}
 
 	/* work out icon (cannot be NULL) */
 	icon = gpk_check_update_get_best_update_icon (cupdate, array);
 	gpk_check_update_set_icon_name (cupdate, icon);
 
-	/* make tooltip */
-	if (status_security->len != 0)
-		g_string_set_size (status_security, status_security->len-1);
 	/* TRANSLATORS: tooltip: how many updates are waiting to be applied */
 	g_string_append_printf (status_tooltip, ngettext ("There is %d update available",
 							  "There are %d updates available", array->len), array->len);
@@ -993,7 +938,7 @@ gpk_check_update_get_updates_finished_cb (GObject *object, GAsyncResult *res, Gp
 
 		/* do we warn the user? */
 		if (security_array->len > 0)
-			gpk_check_update_critical_updates_warning (cupdate, status_security->str, security_array);
+			gpk_check_update_critical_updates_warning (cupdate, security_array);
 		goto out;
 	}
 
@@ -1015,7 +960,7 @@ gpk_check_update_get_updates_finished_cb (GObject *object, GAsyncResult *res, Gp
 
 		/* do we warn the user? */
 		if (security_array->len > 0)
-			gpk_check_update_critical_updates_warning (cupdate, status_security->str, security_array);
+			gpk_check_update_critical_updates_warning (cupdate, security_array);
 		goto out;
 	}
 
@@ -1049,8 +994,6 @@ gpk_check_update_get_updates_finished_cb (GObject *object, GAsyncResult *res, Gp
 out:
 	if (error_code != NULL)
 		g_object_unref (error_code);
-	if (status_security != NULL)
-		g_string_free (status_security, TRUE);
 	if (status_tooltip != NULL)
 		g_string_free (status_tooltip, TRUE);
 	if (security_array != NULL)
@@ -1062,10 +1005,10 @@ out:
 }
 
 /**
- * gpk_check_update_get_active_roles:
+ * gpk_check_update_get_active_roles_for_tids:
  **/
 static PkBitfield
-gpk_check_update_get_active_roles (GpkCheckUpdate *cupdate, gchar **tids)
+gpk_check_update_get_active_roles_for_tids (GpkCheckUpdate *cupdate, gchar **tids)
 {
 	PkRoleEnum role;
 	PkBitfield roles = 0;
@@ -1092,6 +1035,22 @@ out:
 	return roles;
 }
 
+/**
+ * gpk_check_update_get_active_roles:
+ **/
+static PkBitfield
+gpk_check_update_get_active_roles (GpkCheckUpdate *cupdate)
+{
+	PkBitfield roles;
+	gchar **tids;
+
+	/* get active transaction id's */
+	tids = pk_transaction_list_get_ids (cupdate->priv->tlist);
+	roles = gpk_check_update_get_active_roles_for_tids (cupdate, tids);
+	g_strfreev (tids);
+	return roles;
+}
+
 /**
  * gpk_check_update_query_updates:
  **/
@@ -1099,12 +1058,10 @@ static void
 gpk_check_update_query_updates (GpkCheckUpdate *cupdate)
 {
 	PkBitfield roles;
-	gchar **tids;
 	g_return_if_fail (GPK_IS_CHECK_UPDATE (cupdate));
 
 	/* No point if we are already updating */
-	tids = pk_transaction_list_get_ids (cupdate->priv->tlist);
-	roles = gpk_check_update_get_active_roles (cupdate, tids);
+	roles = gpk_check_update_get_active_roles (cupdate);
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_GET_UPDATES) ||
 	    pk_bitfield_contain (roles, PK_ROLE_ENUM_UPDATE_PACKAGES) ||
 	    pk_bitfield_contain (roles, PK_ROLE_ENUM_UPDATE_SYSTEM)) {
@@ -1116,7 +1073,6 @@ gpk_check_update_query_updates (GpkCheckUpdate *cupdate)
 	pk_client_get_updates_async (PK_CLIENT(cupdate->priv->task), PK_FILTER_ENUM_NONE, cupdate->priv->cancellable, NULL, NULL,
 				     (GAsyncReadyCallback) gpk_check_update_get_updates_finished_cb, cupdate);
 out:
-	g_strfreev (tids);
 	return;
 }
 
@@ -1194,7 +1150,7 @@ gpk_check_update_transaction_list_changed_cb (PkControl *control, gchar **transa
 	g_return_if_fail (GPK_IS_CHECK_UPDATE (cupdate));
 
 	/* inhibit icon if we are updating */
-	roles = gpk_check_update_get_active_roles (cupdate, transaction_ids);
+	roles = gpk_check_update_get_active_roles_for_tids (cupdate, transaction_ids);
 	cupdate->priv->icon_inhibit_update_in_progress =
 		(pk_bitfield_contain (roles, PK_ROLE_ENUM_UPDATE_SYSTEM) ||
 		 pk_bitfield_contain (roles, PK_ROLE_ENUM_UPDATE_PACKAGES));
@@ -1247,10 +1203,21 @@ out:
 static void
 gpk_check_update_auto_refresh_cache_cb (GpkAutoRefresh *arefresh, GpkCheckUpdate *cupdate)
 {
+	PkBitfield roles;
+
 	g_return_if_fail (GPK_IS_CHECK_UPDATE (cupdate));
 
+	/* No point if we are already updating */
+	roles = gpk_check_update_get_active_roles (cupdate);
+	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_REFRESH_CACHE)) {
+		egg_debug ("Not refreshing cache as already in progress");
+		goto out;
+	}
+
 	pk_client_refresh_cache_async (PK_CLIENT(cupdate->priv->task), TRUE, cupdate->priv->cancellable, NULL, NULL,
 				       (GAsyncReadyCallback) gpk_check_update_refresh_cache_finished_cb, cupdate);
+out:
+	return;
 }
 
 /**
@@ -1376,12 +1343,10 @@ static void
 gpk_check_update_auto_get_upgrades_cb (GpkAutoRefresh *arefresh, GpkCheckUpdate *cupdate)
 {
 	PkBitfield roles;
-	gchar **tids;
 	g_return_if_fail (GPK_IS_CHECK_UPDATE (cupdate));
 
 	/* No point if we are already updating */
-	tids = pk_transaction_list_get_ids (cupdate->priv->tlist);
-	roles = gpk_check_update_get_active_roles (cupdate, tids);
+	roles = gpk_check_update_get_active_roles (cupdate);
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_GET_DISTRO_UPGRADES)) {
 		egg_debug ("Not checking for upgrades as already in progress");
 		goto out;
@@ -1391,7 +1356,6 @@ gpk_check_update_auto_get_upgrades_cb (GpkAutoRefresh *arefresh, GpkCheckUpdate
 	pk_client_get_distro_upgrades_async (PK_CLIENT(cupdate->priv->task), cupdate->priv->cancellable, NULL, NULL,
 					     (GAsyncReadyCallback) gpk_check_update_get_distro_upgrades_finished_cb, cupdate);
 out:
-	g_strfreev (tids);
 	return;
 }
 
@@ -1465,7 +1429,6 @@ gpk_check_update_init (GpkCheckUpdate *cupdate)
 {
 	gboolean ret;
 	PkBitfield roles;
-	gchar **tids;
 
 	cupdate->priv = GPK_CHECK_UPDATE_GET_PRIVATE (cupdate);
 
@@ -1507,7 +1470,7 @@ gpk_check_update_init (GpkCheckUpdate *cupdate)
 			  G_CALLBACK (gpk_cupdate_connection_changed_cb), cupdate);
 
 	/* use an asynchronous query object */
-	cupdate->priv->task = PK_TASK(pk_task_new ());
+	cupdate->priv->task = PK_TASK (gpk_task_new ());
 	g_object_set (cupdate->priv->task,
 		      "background", TRUE,
 		      NULL);
@@ -1525,10 +1488,8 @@ gpk_check_update_init (GpkCheckUpdate *cupdate)
 	/* we need the task list so we can hide the update icon when we are doing the update */
 	cupdate->priv->tlist = pk_transaction_list_new ();
 
-	tids = pk_transaction_list_get_ids (cupdate->priv->tlist);
-	roles = gpk_check_update_get_active_roles (cupdate, tids);
-	g_strfreev (tids);
 	/* coldplug update in progress */
+	roles = gpk_check_update_get_active_roles (cupdate);
 	cupdate->priv->icon_inhibit_update_in_progress =
 		(pk_bitfield_contain (roles, PK_ROLE_ENUM_UPDATE_SYSTEM) ||
 		 pk_bitfield_contain (roles, PK_ROLE_ENUM_UPDATE_PACKAGES));
